@@ -36,16 +36,17 @@ except ImportError:
     rpmutils_present = False
 import re
 import shutil
-import six
-import six.moves.configparser as ConfigParser
 import subprocess
 import tempfile
 
+import six
+import six.moves.configparser as ConfigParser
+
 
 # Override BOTO_CONFIG, which makes boto look only at the specified
 # config file, instead of the default locations
 os.environ['BOTO_CONFIG'] = '/var/lib/heat-cfntools/cfn-boto-cfg'
-from boto import cloudformation
+from boto import cloudformation  # noqa
 
 
 LOG = logging.getLogger(__name__)
@@ -72,11 +73,17 @@ def parse_creds_file(path='/etc/cfn/cfn-credentials'):
     return creds
 
 
+class InvalidCredentialsException(Exception):
+    def __init__(self, credential_file):
+        super(Exception, self).__init__("invalid credentials file %s" %
+                                        credential_file)
+
+
 class HupConfig(object):
     def __init__(self, fp_list):
-        self.config = ConfigParser.SafeConfigParser()
+        self.config = ConfigParser.ConfigParser()
         for fp in fp_list:
-            self.config.readfp(fp)
+            self.config.read_file(fp)
 
         self.load_main_section()
 
@@ -98,8 +105,7 @@ class HupConfig(object):
             with open(self.credential_file) as f:
                 self.credentials = f.read()
         except Exception:
-            raise Exception("invalid credentials file %s" %
-                            self.credential_file)
+            raise InvalidCredentialsException(self.credential_file)
 
         # optional values
         try:
@@ -113,8 +119,8 @@ class HupConfig(object):
             self.interval = 10
 
     def __str__(self):
-        return '{stack: %s, credential_file: %s, region: %s, interval:%d}' % \
-            (self.stack, self.credential_file, self.region, self.interval)
+        return ('{stack: %s, credential_file: %s, region: %s, interval:%d}' %
+                (self.stack, self.credential_file, self.region, self.interval))
 
     def unique_resources_get(self):
         resources = []
@@ -138,20 +144,19 @@ class Hook(object):
         return sp[1]
 
     def event(self, ev_name, ev_object, ev_resource):
-        if self.resource_name_get() == ev_resource and \
-                ev_name in self.triggers:
+        if (self.resource_name_get() == ev_resource and
+                ev_name in self.triggers):
             CommandRunner(self.action, shell=True).run(user=self.runas)
         else:
             LOG.debug('event: {%s, %s, %s} did not match %s' %
                       (ev_name, ev_object, ev_resource, self.__str__()))
 
     def __str__(self):
-        return '{%s, %s, %s, %s, %s}' % \
-            (self.name,
-             self.triggers,
-             self.path,
-             self.runas,
-             self.action)
+        return '{%s, %s, %s, %s, %s}' % (self.name,
+                                         self.triggers,
+                                         self.path,
+                                         self.runas,
+                                         self.action)
 
 
 class ControlledPrivilegesFailureException(Exception):
@@ -306,7 +311,7 @@ class RpmHelper(object):
         Arguments:
             pkg -- A package name
         """
-        cmd = "rpm -q --queryformat '%{VERSION}-%{RELEASE}' %s" % pkg
+        cmd = "rpm -q --queryformat '%%{VERSION}-%%{RELEASE}' %s" % pkg
         command = CommandRunner(cmd).run()
         return command.stdout
 
@@ -401,7 +406,7 @@ class RpmHelper(object):
         LOG.info("Installing packages: %s" % cmd)
         command = CommandRunner(cmd).run()
         if command.status:
-            LOG.warn("Failed to install packages: %s" % cmd)
+            LOG.warning("Failed to install packages: %s" % cmd)
 
     @classmethod
     def downgrade(cls, packages, rpms=True, zypper=False, dnf=False):
@@ -429,21 +434,21 @@ class RpmHelper(object):
             LOG.info("Downgrading packages: %s", cmd)
             command = CommandRunner(cmd).run()
             if command.status:
-                LOG.warn("Failed to downgrade packages: %s" % cmd)
+                LOG.warning("Failed to downgrade packages: %s" % cmd)
         elif dnf:
             cmd = ['dnf', '-y', 'downgrade']
             cmd.extend(packages)
             LOG.info("Downgrading packages: %s", cmd)
             command = CommandRunner(cmd).run()
             if command.status:
-                LOG.warn("Failed to downgrade packages: %s" % cmd)
+                LOG.warning("Failed to downgrade packages: %s" % cmd)
         else:
             cmd = ['yum', '-y', 'downgrade']
             cmd.extend(packages)
             LOG.info("Downgrading packages: %s" % cmd)
             command = CommandRunner(cmd).run()
             if command.status:
-                LOG.warn("Failed to downgrade packages: %s" % cmd)
+                LOG.warning("Failed to downgrade packages: %s" % cmd)
 
 
 class PackagesHandler(object):
@@ -457,13 +462,17 @@ class PackagesHandler(object):
         p1_name = pkg1[0]
         p2_name = pkg2[0]
         if p1_name in order and p2_name in order:
-            return cmp(order.index(p1_name), order.index(p2_name))
+            i1 = order.index(p1_name)
+            i2 = order.index(p2_name)
+            return (i1 > i2) - (i1 < i2)
         elif p1_name in order:
             return -1
         elif p2_name in order:
             return 1
         else:
-            return cmp(p1_name.lower(), p2_name.lower())
+            n1 = p1_name.lower()
+            n2 = p2_name.lower()
+            return (n1 > n2) - (n1 < n2)
 
     def __init__(self, packages):
         self._packages = packages
@@ -521,7 +530,8 @@ class PackagesHandler(object):
                 # FIXME:print non-error, but skipping pkg
                 pass
             elif not RpmHelper.zypper_package_available(pkg):
-                LOG.warn("Skipping package '%s' - unavailable via zypper", pkg)
+                LOG.warning(
+                    "Skipping package '%s' - unavailable via zypper", pkg)
             elif not ver:
                 installs.append(pkg)
             else:
@@ -566,7 +576,8 @@ class PackagesHandler(object):
                 # FIXME:print non-error, but skipping pkg
                 pass
             elif not RpmHelper.dnf_package_available(pkg):
-                LOG.warn("Skipping package '%s'. Not available via yum" % pkg)
+                LOG.warning(
+                    "Skipping package '%s'. Not available via yum" % pkg)
             elif not ver:
                 installs.append(pkg)
             else:
@@ -622,7 +633,8 @@ class PackagesHandler(object):
                 # FIXME:print non-error, but skipping pkg
                 pass
             elif not RpmHelper.yum_package_available(pkg):
-                LOG.warn("Skipping package '%s'. Not available via yum" % pkg)
+                LOG.warning(
+                    "Skipping package '%s'. Not available via yum" % pkg)
             elif not ver:
                 installs.append(pkg)
             else:
@@ -649,7 +661,7 @@ class PackagesHandler(object):
           * if a different version of the package is installed, overwrite it
           * if the package isn't installed, install it
         """
-        #FIXME: handle rpm installs
+        # FIXME(asalkeld): handle rpm installs
         pass
 
     def _handle_apt_packages(self, packages):
@@ -702,7 +714,7 @@ class PackagesHandler(object):
         for manager, package_entries in packages:
             handler = self._package_handler(manager)
             if not handler:
-                LOG.warn("Skipping invalid package type: %s" % manager)
+                LOG.warning("Skipping invalid package type: %s" % manager)
             else:
                 handler(self, package_entries)
 
@@ -731,8 +743,8 @@ class FilesHandler(object):
                     f.close()
                 else:
                     f = open(dest, 'w+')
-                    f.write(json.dumps(meta['content'], indent=4)
-                     .encode('UTF-8'))
+                    f.write(json.dumps(meta['content'],
+                                       indent=4).encode('UTF-8'))
                     f.close()
             elif 'source' in meta:
                 CommandRunner(['curl', '-o', dest, meta['source']]).run()
@@ -836,7 +848,7 @@ class SourcesHandler(object):
 
     def _apply_source(self, dest, url):
         cmd = self._apply_source_cmd(dest, url)
-        #FIXME bug 1498298
+        # FIXME bug 1498298
         if cmd != '':
             runner = CommandRunner(cmd, shell=True)
             runner.run()
@@ -934,11 +946,11 @@ class ServicesHandler(object):
             command = handler(self, service, "status")
             running = command.status == 0
             if ensure_running and not running:
-                LOG.warn("Restarting service %s" % service)
+                LOG.warning("Restarting service %s" % service)
                 start_cmd = handler(self, service, "start")
                 if start_cmd.status != 0:
                     LOG.warning('Service %s did not start. STDERR: %s' %
-                               (service, start_cmd.stderr))
+                                (service, start_cmd.stderr))
                 for h in self.hooks:
                     h.event('service.restarted', service, self.resource)
 
@@ -969,7 +981,7 @@ class ServicesHandler(object):
         for manager, service_entries in self._services.items():
             handler = self._service_handler(manager)
             if not handler:
-                LOG.warn("Skipping invalid service type: %s" % manager)
+                LOG.warning("Skipping invalid service type: %s" % manager)
             else:
                 self._initialize_services(handler, service_entries)
 
@@ -980,7 +992,7 @@ class ServicesHandler(object):
         for manager, service_entries in self._services.items():
             handler = self._service_handler(manager)
             if not handler:
-                LOG.warn("Skipping invalid service type: %s" % manager)
+                LOG.warning("Skipping invalid service type: %s" % manager)
             else:
                 self._monitor_services(handler, service_entries)
 
@@ -1100,11 +1112,8 @@ class CommandsHandler(object):
         if "command" in properties:
             try:
                 command = properties["command"]
-                #FIXME bug 1498300
-                if isinstance(command, list):
-                    escape = lambda x: '"%s"' % x.replace('"', '\\"')
-                    command = ' '.join(map(escape, command))
-                command = CommandRunner(command, shell=True)
+                shell = isinstance(command, six.string_types)
+                command = CommandRunner(command, shell=shell)
                 command.run('root', cwd, env)
                 command_status = command.status
             except OSError as e:
@@ -1120,8 +1129,8 @@ class CommandsHandler(object):
         if command_status == 0:
             LOG.info("%s has been successfully executed" % command_label)
         else:
-            if "ignoreErrors" in properties and \
-               to_boolean(properties["ignoreErrors"]):
+            if ("ignoreErrors" in properties and
+                    to_boolean(properties["ignoreErrors"])):
                 LOG.info("%s has failed (status=%d). Explicit ignoring"
                          % (command_label, command_status))
             else:
@@ -1154,19 +1163,19 @@ class GroupsHandler(object):
         if command_status == 0:
             LOG.info("%s has been successfully created" % group)
         elif command_status == 9:
-            LOG.error("An error occured creating %s group : " %
+            LOG.error("An error occurred creating %s group : " %
                       group + "group name not unique")
         elif command_status == 4:
-            LOG.error("An error occured creating %s group : " %
+            LOG.error("An error occurred creating %s group : " %
                       group + "GID not unique")
         elif command_status == 3:
-            LOG.error("An error occured creating %s group : " %
+            LOG.error("An error occurred creating %s group : " %
                       group + "GID not valid")
         elif command_status == 2:
-            LOG.error("An error occured creating %s group : " %
+            LOG.error("An error occurred creating %s group : " %
                       group + "Invalid syntax")
         else:
-            LOG.error("An error occured creating %s group" % group)
+            LOG.error("An error occurred creating %s group" % group)
 
 
 class UsersHandler(object):
@@ -1198,8 +1207,8 @@ class UsersHandler(object):
             groups = ','.join(properties["groups"])
             cmd.extend(['--groups', groups])
 
-        #Users are created as non-interactive system users with a shell
-        #of /sbin/nologin. This is by design and cannot be modified.
+        # Users are created as non-interactive system users with a shell
+        # of /sbin/nologin. This is by design and cannot be modified.
         cmd.extend(['--shell', '/sbin/nologin'])
 
         command = CommandRunner(cmd)
@@ -1209,22 +1218,22 @@ class UsersHandler(object):
         if command_status == 0:
             LOG.info("%s has been successfully created" % user)
         elif command_status == 9:
-            LOG.error("An error occured creating %s user : " %
+            LOG.error("An error occurred creating %s user : " %
                       user + "user name not unique")
         elif command_status == 6:
-            LOG.error("An error occured creating %s user : " %
+            LOG.error("An error occurred creating %s user : " %
                       user + "group does not exist")
         elif command_status == 4:
-            LOG.error("An error occured creating %s user : " %
+            LOG.error("An error occurred creating %s user : " %
                       user + "UID not unique")
         elif command_status == 3:
-            LOG.error("An error occured creating %s user : " %
+            LOG.error("An error occurred creating %s user : " %
                       user + "Invalid argument")
         elif command_status == 2:
-            LOG.error("An error occured creating %s user : " %
+            LOG.error("An error occurred creating %s user : " %
                       user + "Invalid syntax")
         else:
-            LOG.error("An error occured creating %s user" % user)
+            LOG.error("An error occurred creating %s user" % user)
 
 
 class MetadataServerConnectionError(Exception):
@@ -1256,7 +1265,7 @@ class Metadata(object):
         self._has_changed = False
 
     def remote_metadata(self):
-        """Connect to the metadata server and retreive the metadata."""
+        """Connect to the metadata server and retrieve the metadata."""
 
         if self.credentials_file:
             credentials = parse_creds_file(self.credentials_file)
@@ -1346,7 +1355,8 @@ class Metadata(object):
             try:
                 self._data = self.remote_metadata()
             except MetadataServerConnectionError as ex:
-                LOG.warn("Unable to retrieve remote metadata : %s" % str(ex))
+                LOG.warning(
+                    "Unable to retrieve remote metadata : %s" % str(ex))
 
                 # If reading remote metadata fails, we fall-back on local files
                 # in order to get the most up-to-date version, we try:
@@ -1363,8 +1373,8 @@ class Metadata(object):
                     try:
                         fd = open(filepath)
                     except IOError:
-                        LOG.warn("Unable to open local metadata : %s" %
-                                 filepath)
+                        LOG.warning("Unable to open local metadata : %s" %
+                                    filepath)
                         continue
                     else:
                         LOG.info("Opened local metadata %s" % filepath)
@@ -1392,8 +1402,8 @@ class Metadata(object):
                         pass
                     lm.close()
             except IOError:
-                LOG.warn("Unable to open local metadata : %s" %
-                         metadata_file)
+                LOG.warning("Unable to open local metadata : %s" %
+                            metadata_file)
                 continue
 
         if self._metadata != last_data:
@@ -1405,8 +1415,8 @@ class Metadata(object):
             try:
                 os.makedirs(cache_dir, mode=0o700)
             except IOError as e:
-                LOG.warn('could not create metadata cache dir %s [%s]' %
-                         (cache_dir, e))
+                LOG.warning('could not create metadata cache dir %s [%s]' %
+                            (cache_dir, e))
                 return
         # save current metadata to file
         tmp_dir = os.path.dirname(last_path)
@@ -1426,9 +1436,10 @@ class Metadata(object):
         return json.dumps(self._metadata)
 
     def display(self, key=None):
-        """Print the metadata to the standard output stream. By default the
-        full metadata is displayed but the ouptut can be limited to a specific
-        with the <key> argument.
+        """Print the metadata to the standard output stream.
+
+        By default the full metadata is displayed but the ouptut can be limited
+        to a specific with the <key> argument.
 
         Arguments:
             key -- the metadata's key to display, nested keys can be specified
@@ -1470,9 +1481,9 @@ class Metadata(object):
 
     def _is_valid_metadata(self):
         """Should find the AWS::CloudFormation::Init json key."""
-        is_valid = self._metadata and \
-            self._init_key in self._metadata and \
-            self._metadata[self._init_key]
+        is_valid = (self._metadata and
+                    self._init_key in self._metadata and
+                    self._metadata[self._init_key])
         if is_valid:
             self._metadata = self._metadata[self._init_key]
         return is_valid
